@@ -33,9 +33,10 @@
 #include <mythtv/mythcontext.h>
 
 #include "glsingleview.h"
+#include "galleryutil.h"
 
 GLSDialog::GLSDialog(QSqlDatabase *db, const ThumbList& itemList,
-                     int pos, bool slideShow, MythMainWindow *parent,
+                     int pos, int slideShow, MythMainWindow *parent,
                      const char *name)
     : MythDialog(parent, name)
 {
@@ -57,12 +58,14 @@ void GLSDialog::closeEvent(QCloseEvent *e)
 }
 
 GLSingleView::GLSingleView(QSqlDatabase *db, ThumbList itemList,
-                           int pos, bool slideShow, QWidget *parent)
+                           int pos, int slideShow, QWidget *parent)
     : QGLWidget(parent)
 {
     m_db       = db;
     m_pos      = pos;
     m_itemList = itemList;
+    m_movieState  = 0;
+    m_slideShow = slideShow;
 
     // --------------------------------------------------------------------
 
@@ -141,9 +144,14 @@ GLSingleView::GLSingleView(QSqlDatabase *db, ThumbList itemList,
     m_timer = new QTimer(this);
     connect(m_timer, SIGNAL(timeout()),
             SLOT(slotTimeOut()));
+
+    if (slideShow > 1)
+        randomFrame();
+
     if (slideShow) {
         m_running = true;
         m_timer->start(m_tmout, true);
+        gContext->DisableScreensaver();
     }
 }
 
@@ -194,6 +202,23 @@ void GLSingleView::resizeGL( int w, int h )
 
 void GLSingleView::paintGL()
 {
+    if (m_movieState > 0)
+    {
+        if (m_movieState == 1)
+        {
+            m_movieState = 2;
+            ThumbItem* item = m_itemList.at(m_pos);
+            QString cmd = gContext->GetSetting("GalleryMoviePlayerCmd");
+            cmd.replace("%s", item->path);
+            myth_system(cmd);
+            if (!m_running)
+            {
+                close();
+            }
+        }
+        return;
+    }
+ 
     glDisable(GL_DEPTH_TEST);
 
     glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
@@ -223,6 +248,7 @@ void GLSingleView::keyPressEvent(QKeyEvent *e)
     bool wasRunning = m_running;
     m_timer->stop();
     m_running = false;
+    gContext->RestoreScreensaver();
     m_effectRunning = false;
     m_tmout = m_delay * 1000;
 
@@ -378,6 +404,7 @@ void GLSingleView::keyPressEvent(QKeyEvent *e)
 
     if (m_running) {
         m_timer->start(m_tmout, true);
+        gContext->DisableScreensaver();
     }
     
     if (handled) {
@@ -463,6 +490,18 @@ void GLSingleView::advanceFrame()
     m_curr      = (m_curr == 0) ? 1 : 0;
 }
 
+void GLSingleView::randomFrame()
+{
+    int newframe;
+    if(m_itemList.count() > 1){
+        while((newframe = (int)(rand()/(RAND_MAX+1.0) * m_itemList.count())) ==m_pos);
+        m_pos = newframe;
+    }
+
+    m_tex1First = !m_tex1First;
+    m_curr      = (m_curr == 0) ? 1 : 0;
+}
+
 void GLSingleView::retreatFrame()
 {
     m_pos--;
@@ -475,6 +514,7 @@ void GLSingleView::retreatFrame()
 
 void GLSingleView::loadImage()
 {
+    m_movieState = 0;
     ThumbItem *item = m_itemList.at(m_pos);
     if (!item) {
         std::cerr << "GLSingleView: The impossible happened. No item at "
@@ -482,6 +522,11 @@ void GLSingleView::loadImage()
         return;
     }
 
+    if (GalleryUtil::isMovie(item->path)) {
+        m_movieState = 1;
+        return;
+    }
+
     QImage image(item->path);
     if (!image.isNull()) {
 
@@ -491,15 +536,7 @@ void GLSingleView::loadImage()
         t.item     = item;
         t.angle    = 0;
 
-        QString queryStr = "SELECT angle FROM gallerymetadata WHERE "
-                           "image=\"" + item->path + "\";";
-        QSqlQuery query = m_db->exec(queryStr);
-        
-        if (query.isActive()  && query.numRowsAffected() > 0) 
-        {
-            query.next();
-            t.angle = query.value(0).toInt(); 
-        }
+        t.angle = item->GetRotationAngle(m_db);
 
         t.width  = image.width();
         t.height = image.height();
@@ -520,7 +557,7 @@ void GLSingleView::loadImage()
         QImage tex = convertToGLFormat(image);
 
         if (t.tex) {
-            glDeleteTextures(a, &t.tex);
+            glDeleteTextures(1, &t.tex);
         }
 
         /* create the texture */
@@ -550,10 +587,7 @@ void GLSingleView::rotate(int angle)
 
     ThumbItem *item = m_itemList.at(m_pos);
     if (item) {
-        QString queryStr = "REPLACE INTO gallerymetadata SET image=\"" +
-                           item->path + "\", angle=" + 
-                           QString::number(ang) + ";";
-        m_db->exec(queryStr);
+        item->SetRotationAngle(ang, m_db);
 
         // Delete thumbnail for this
         if (item->pixmap)
@@ -1319,12 +1353,26 @@ void GLSingleView::slotTimeOut()
             if (m_effectRandom)
                 m_effectMethod = getRandomEffect();
 
-            advanceFrame();
-            loadImage();
+            if (m_slideShow > 1)
+                randomFrame();
+            else
+                advanceFrame();
 
-            m_tmout = 10;
-            m_effectRunning = true;
-            m_i = 0;
+            bool wasMovie = m_movieState > 0;
+            loadImage();
+            bool isMovie = m_movieState > 0;
+            // If transitioning to/from a movie, don't do an effect,
+            // and shorten timeout
+            if (wasMovie || isMovie)
+            {
+                m_tmout = 1;
+            }
+            else
+            {
+                m_tmout = 10;
+                m_effectRunning = true;
+                m_i = 0;
+            }
         }
     }
 
