@@ -31,18 +31,48 @@ extern "C" {
 #include <mythtv/uilistbtntype.h>
 #include <mythtv/xmlparse.h>
 #include <mythtv/dialogbox.h>
+#include <mythtv/mythdbcon.h>
 
+#include "galleryutil.h"
 #include "gallerysettings.h"
 #include "thumbgenerator.h"
 #include "singleview.h"
 #include "iconview.h"
 
 #include "config.h"
+#include "constants.h"
 
 #ifdef OPENGL_SUPPORT
 #include "glsingleview.h"
 #endif
 
+
+int ThumbItem::GetRotationAngle(QSqlDatabase *db)
+{
+    MSqlQuery query(QString::null, db);
+    query.prepare("SELECT angle FROM gallerymetadata WHERE "
+                  "image = :PATH ;");
+    query.bindValue(":PATH", path.utf8());
+
+    if (query.exec() && query.isActive() && query.size() > 0)
+    {
+        query.next();
+        return query.value(0).toInt();
+    }
+
+    return GalleryUtil::getNaturalRotation(path);
+}
+
+void ThumbItem::SetRotationAngle(int angle, QSqlDatabase *db)
+{
+    MSqlQuery query(QString::null, db);
+    query.prepare("REPLACE INTO gallerymetadata SET image = :IMAGE , "
+                  "angle = :ANGLE ;");
+    query.bindValue(":IMAGE", path.utf8());
+    query.bindValue(":ANGLE", angle);
+    query.exec();
+}
+
 IconView::IconView(QSqlDatabase *db, const QString& galleryDir,
                    MythMainWindow* parent, const char* name )
     : MythDialog(parent, name)
@@ -307,16 +337,17 @@ void IconView::keyPressEvent(QKeyEvent *e)
                     std::cerr << "The impossible happened" << std::endl;
                     break;
                 }
-
+        
+                QFileInfo fi(item->path);
                 if (item->isDir) {
                     loadDirectory(item->path);
                     handled = true;
                 }
                 else {
-
+          
                     handled = true;
                     
-                    bool slideShow = (action == "PLAY");
+                    int slideShow = (action == "PLAY")?1:0;
 #ifdef OPENGL_SUPPORT
                     int useOpenGL = gContext->GetNumSetting("SlideshowUseOpenGL");
                     if (useOpenGL) {
@@ -398,16 +429,7 @@ void IconView::customEvent(QCustomEvent *e)
             delete item->pixmap;
         item->pixmap = 0;
 
-        QString queryStr = "SELECT angle FROM gallerymetadata WHERE image =\"" +
-                           item->path + "\";";
-        QSqlQuery query = m_db->exec(queryStr);
-        int rotateAngle = 0;
-
-        if (query.isActive() && query.numRowsAffected() > 0) 
-        {
-            query.next();
-            rotateAngle = query.value(0).toInt();
-        }
+        int rotateAngle = item->GetRotationAngle(m_db);
 
         if (rotateAngle)
         {
@@ -416,7 +438,6 @@ void IconView::customEvent(QCustomEvent *e)
             td->thumb = td->thumb.xForm(matrix);
         }
 
-        item->pixmap = new QPixmap(td->thumb);
 
         int pos = m_itemList.find(item);
         
@@ -487,6 +508,8 @@ void IconView::loadTheme()
     UIListBtnTypeItem* item;
     item = new UIListBtnTypeItem(m_menuType, tr("SlideShow"));
     item->setData(new Action(&IconView::actionSlideShow));
+    item = new UIListBtnTypeItem(m_menuType, tr("Random"));
+    item->setData(new Action(&IconView::actionRandomShow));
     item = new UIListBtnTypeItem(m_menuType, tr("Rotate CW"));
     item->setData(new Action(&IconView::actionRotateCW));
     item = new UIListBtnTypeItem(m_menuType, tr("Rotate CCW"));
@@ -585,8 +608,7 @@ void IconView::loadDirectory(const QString& dir)
     if (!cdir.exists())
         d.mkdir(".thumbcache");
 
-    d.setNameFilter("*.jpg *.JPG *.jpeg *.JPEG *.png *.PNG *.tiff *.TIFF "
-                    "*.tif *.TIF *.bmp *.BMP *.gif *.GIF *.ppm *.PPM");
+    d.setNameFilter(MEDIA_FILENAMES);
     d.setSorting(QDir::Name | QDir::DirsFirst | QDir::IgnoreCase);
 
     d.setMatchAllDirs(true);
@@ -623,6 +645,11 @@ void IconView::loadDirectory(const QString& dir)
         m_thumbGen->addFile(item->name);
     }
 
+    if (!m_thumbGen->running())
+    {
+        m_thumbGen->start();
+    }
+
     m_lastRow = QMAX((int)ceilf((float)m_itemList.count()/(float)m_nCols)-1,0);
     m_lastCol = QMAX(m_itemList.count()-m_lastRow*m_nCols-1,0);
 }
@@ -671,19 +698,16 @@ void IconView::loadThumbnail(ThumbItem *item)
                                   (int)(m_thumbW-10*wmult),
                                   QImage::ScaleMax);
         int rotateAngle = 0;
-        
-        QString queryStr = "SELECT angle FROM gallerymetadata WHERE image =\"" +
-                           item->path + "\";";
-        QSqlQuery query = m_db->exec(queryStr);
-        if (query.isActive() && query.numRowsAffected() > 0) {
-            query.next();
-            rotateAngle = query.value(0).toInt();
+
+        rotateAngle = item->GetRotationAngle(m_db);
+
+        if (rotateAngle != 0)
+        {
+            QWMatrix matrix;
+            matrix.rotate(rotateAngle);
+            image = image.xForm(matrix);
         }
 
-        QWMatrix matrix;
-        matrix.rotate(rotateAngle);
-        image = image.xForm(matrix);
-        
         item->pixmap = new QPixmap(image);
     }
 }
@@ -764,28 +788,15 @@ void IconView::actionRotateCW()
     if (!item || item->isDir)
         return;
 
-    int rotAngle = 0;
+    int rotAngle = item->GetRotationAngle(m_db);
     
-    QString queryStr = "SELECT angle FROM gallerymetadata WHERE "
-                       "image=\"" + item->path + "\";";
-    QSqlQuery query = m_db->exec(queryStr);
-			
-    if (query.isActive()  && query.numRowsAffected() > 0) 
-    {
-        query.next();
-        rotAngle = query.value(0).toInt();
-    }
-
     rotAngle += 90;
     if (rotAngle >= 360)
         rotAngle -= 360;
     if (rotAngle < 0)
         rotAngle += 360;
 
-    queryStr = "REPLACE INTO gallerymetadata SET image=\"" +
-               item->path + "\", angle=" + 
-               QString::number(rotAngle) + ";";
-    query = m_db->exec(queryStr);
+    item->SetRotationAngle(rotAngle, m_db);
 
     if (item->pixmap) {
         delete item->pixmap;
@@ -800,28 +811,15 @@ void IconView::actionRotateCCW()
     if (!item || item->isDir)
         return;
 
-    int rotAngle = 0;
+    int rotAngle = item->GetRotationAngle(m_db);;
     
-    QString queryStr = "SELECT angle FROM gallerymetadata WHERE "
-                       "image=\"" + item->path + "\";";
-    QSqlQuery query = m_db->exec(queryStr);
-			
-    if (query.isActive()  && query.numRowsAffected() > 0) 
-    {
-        query.next();
-        rotAngle = query.value(0).toInt();
-    }
-
     rotAngle -= 90;
     if (rotAngle >= 360)
         rotAngle -= 360;
     if (rotAngle < 0)
         rotAngle += 360;
 
-    queryStr = "REPLACE INTO gallerymetadata SET image=\"" +
-               item->path + "\", angle=" + 
-               QString::number(rotAngle) + ";";
-    query = m_db->exec(queryStr);
+    item->SetRotationAngle(rotAngle, m_db);
 
     if (item->pixmap) {
         delete item->pixmap;
@@ -842,7 +840,7 @@ void IconView::actionSlideShow()
     int useOpenGL = gContext->GetNumSetting("SlideshowUseOpenGL");
     if (useOpenGL) {
         if (QGLFormat::hasOpenGL()) {
-            GLSDialog gv(m_db, m_itemList, pos, true,
+            GLSDialog gv(m_db, m_itemList, pos, 1,
                          gContext->GetMainWindow());
             gv.exec();
         }
@@ -855,7 +853,39 @@ void IconView::actionSlideShow()
     else 
 #endif
     {
-        SingleView sv(m_db, m_itemList, pos, true,
+        SingleView sv(m_db, m_itemList, pos, 1,
+                      gContext->GetMainWindow());
+        sv.exec();
+    }                         
+}
+
+void IconView::actionRandomShow()
+{
+    ThumbItem* item = m_itemList.at(m_currRow * m_nCols +
+                                    m_currCol);
+    if (!item || item->isDir)
+        return;
+
+    int pos = m_currRow * m_nCols + m_currCol;
+
+#ifdef OPENGL_SUPPORT
+    int useOpenGL = gContext->GetNumSetting("SlideshowUseOpenGL");
+    if (useOpenGL) {
+        if (QGLFormat::hasOpenGL()) {
+            GLSDialog gv(m_db, m_itemList, pos, 2,
+                         gContext->GetMainWindow());
+            gv.exec();
+        }
+        else {
+            MythPopupBox::showOkPopup(gContext->GetMainWindow(),
+                                      tr("Error"),
+                                      tr("Sorry: OpenGL support not available"));
+        }
+    }
+    else 
+#endif
+    {
+        SingleView sv(m_db, m_itemList, pos, 2,
                       gContext->GetMainWindow());
         sv.exec();
     }                         
@@ -881,8 +911,9 @@ void IconView::actionImport()
     QStringList paths = QStringList::split(":",
                                            gContext->GetSetting("GalleryImportDirs"));
 
+    // Makes import directory samba/windows friendly (no colon)
     QString idirname(m_currDir + "/" +
-                     (QDateTime::currentDateTime()).toString(Qt::ISODate));
+                     (QDateTime::currentDateTime()).toString("yyyy-MM-dd_hh-mm-ss"));
     importdir.mkdir(idirname);
     importdir.setPath(idirname);
 
@@ -926,6 +957,11 @@ void IconView::actionImport()
     m_itemList.append(item);
     m_itemDict.insert(item->name, item);
     m_thumbGen->addFile(item->name);
+
+    if (!m_thumbGen->running())
+    {
+        m_thumbGen->start();
+    }
 }
 
 void IconView::importFromDir(const QString &fromDir, const QString &toDir)
@@ -935,8 +971,7 @@ void IconView::importFromDir(const QString &fromDir, const QString &toDir)
     if (!d.exists())
         return;
 
-    d.setNameFilter("*.jpg *.JPG *.jpeg *.JPEG *.png *.PNG *.tiff *.TIFF "
-                    "*.bmp *.BMP *.gif *.GIF");
+    d.setNameFilter(MEDIA_FILENAMES);
     d.setSorting(QDir::Name | QDir::DirsFirst | QDir::IgnoreCase);
     d.setFilter(QDir::Files | QDir::Dirs | QDir::NoSymLinks  | QDir::Readable);
     d.setMatchAllDirs(true);
@@ -960,8 +995,10 @@ void IconView::importFromDir(const QString &fromDir, const QString &toDir)
         } 
         else 
         {
-            cerr << "copying " << fi->absFilePath() << " to " << toDir << "\n";
-            QString cmd = "cp \"" + fi->absFilePath() + "\" \"" + toDir + "\"";
+            cerr << "copying " << fi->absFilePath().local8Bit() 
+                 << " to " << toDir.local8Bit() << "\n";
+            QString cmd = "cp \"" + fi->absFilePath().local8Bit() + 
+                          "\" \"" + toDir.local8Bit() + "\"";
             system(cmd);
         }
     }
