@@ -28,16 +28,19 @@
 #include <mythtv/mythcontext.h>
 
 #include "singleview.h"
+#include "constants.h"
+#include "galleryutil.h"
 
 SingleView::SingleView(QSqlDatabase *db, ThumbList itemList,
-                       int pos, bool slideShow,
+                       int pos, int slideShow,
                        MythMainWindow *parent, const char *name )
     : MythDialog(parent, name)
 {
     m_db        = db;
     m_itemList  = itemList;
     m_pos       = pos;
-    
+    m_slideShow = slideShow;
+
     // --------------------------------------------------------------------
 
     // remove all dirs from m_itemList;
@@ -88,6 +91,7 @@ SingleView::SingleView(QSqlDatabase *db, ThumbList itemList,
 
     // --------------------------------------------------------------------
 
+    m_movieState  = 0;
     m_pixmap      = 0;
     m_rotateAngle = 0;
     m_zoom        = 1;
@@ -112,10 +116,13 @@ SingleView::SingleView(QSqlDatabase *db, ThumbList itemList,
 
     // --------------------------------------------------------------------
 
+    if(slideShow > 1)
+        randomFrame();
     loadImage();
     if (slideShow) {
         m_running = true;
         m_timer->start(m_tmout, true);
+        gContext->DisableScreensaver();
     }
 }
 
@@ -142,7 +149,22 @@ SingleView::~SingleView()
 
 void SingleView::paintEvent(QPaintEvent *)
 {
-    if (!m_effectRunning) {
+    if (m_movieState > 0)
+    {
+        if (m_movieState == 1)
+        {
+            m_movieState = 2;
+            ThumbItem* item = m_itemList.at(m_pos);
+            QString cmd = gContext->GetSetting("GalleryMoviePlayerCmd");
+            cmd.replace("%s", item->path);
+            myth_system(cmd);
+            if (!m_running)
+            {
+                reject();
+            }
+        }
+    }
+    else if (!m_effectRunning) {
         
         QPixmap pix(screenwidth, screenheight);
         pix.fill(this, 0, 0);
@@ -217,6 +239,7 @@ void SingleView::keyPressEvent(QKeyEvent *e)
     bool wasRunning = m_running;
     m_timer->stop();
     m_running = false;
+    gContext->RestoreScreensaver();
     m_effectRunning = false;
     m_tmout = m_delay * 1000;
     if (m_painter && m_painter->isActive())
@@ -381,6 +404,7 @@ void SingleView::keyPressEvent(QKeyEvent *e)
 
     if (m_running) {
         m_timer->start(m_tmout, true);
+        gContext->DisableScreensaver();
     }
 
     if (handled) {
@@ -398,6 +422,15 @@ void SingleView::advanceFrame()
         m_pos = 0;
 }
 
+void SingleView::randomFrame()
+{
+    int newframe;
+    if(m_itemList.count() > 1){
+        while((newframe = (int)(rand()/(RAND_MAX+1.0) * m_itemList.count())) == m_pos);
+        m_pos = newframe;
+    }
+}
+
 void SingleView::retreatFrame()
 {
     m_pos--;
@@ -407,6 +440,7 @@ void SingleView::retreatFrame()
 
 void SingleView::loadImage()
 {
+    m_movieState = 0;
     if (m_pixmap) {
         delete m_pixmap;
         m_pixmap = 0;
@@ -414,34 +448,31 @@ void SingleView::loadImage()
     
     ThumbItem *item = m_itemList.at(m_pos);
     if (item) {
-
+      if (GalleryUtil::isMovie(item->path)) {
+        m_movieState = 1;
+      }
+      else {
         m_image.load(item->path);
-
+        
         if (!m_image.isNull()) {
 
-            QString queryStr = "SELECT angle FROM gallerymetadata WHERE "
-                               "image=\"" + item->path + "\";";
-            QSqlQuery query = m_db->exec(queryStr);
-			
-            if (query.isActive()  && query.numRowsAffected() > 0) 
-            {
-                query.next();
-                m_rotateAngle = query.value(0).toInt();
-                if (m_rotateAngle != 0) {
-                    QWMatrix matrix;
-                    matrix.rotate(m_rotateAngle);
-                    m_image = m_image.xForm(matrix);
-                }
-            }
-
-            m_pixmap = new QPixmap(m_image.smoothScale(screenwidth, screenheight,
-                                                       QImage::ScaleMin));
-
+          m_rotateAngle = item->GetRotationAngle(m_db);
+          
+          if (m_rotateAngle != 0) {
+            QWMatrix matrix;
+            matrix.rotate(m_rotateAngle);
+            m_image = m_image.xForm(matrix);
+          }
+        
+          m_pixmap = new QPixmap(m_image.smoothScale(screenwidth, screenheight,
+                                 QImage::ScaleMin));
+          
         }
-        else 
-            std::cerr << "SingleView: Failed to load image "
-                      << item->path << std::endl;
-    
+      }
+    }
+    else {
+      std::cerr << "SingleView: Failed to load image "
+        << item->path << std::endl;
     }
 }
 
@@ -455,10 +486,7 @@ void SingleView::rotate(int angle)
 
     ThumbItem *item = m_itemList.at(m_pos);
     if (item) {
-        QString queryStr = "REPLACE INTO gallerymetadata SET image=\"" +
-                           item->path + "\", angle=" + 
-                           QString::number(m_rotateAngle) + ";";
-        m_db->exec(queryStr);
+        item->SetRotationAngle(m_rotateAngle, m_db);
 
         // Delete thumbnail for this
         if (item->pixmap)
@@ -1155,14 +1183,27 @@ void SingleView::slotTimeOut()
             if (m_effectRandom)
                 m_effectMethod = getRandomEffect();
 
-            advanceFrame();
-            loadImage();
-            createEffectPix();
+             if (m_slideShow > 1)
+                 randomFrame();
+             else
+                 advanceFrame();
 
-            m_tmout = 10;
-            m_effectRunning = true;
-            m_i = 0;
-        }
+            bool wasMovie = m_movieState > 0;
+            loadImage();
+            bool isMovie = m_movieState > 0;
+            // If transitioning to/from a movie, don't do an effect,
+            // and shorten timeout
+            if (wasMovie || isMovie)
+            {
+                m_tmout = 1;
+            }
+            else {
+                createEffectPix();
+                m_effectRunning = true;
+                m_tmout = 10;
+                m_i = 0;
+            }
+       }   
     }
 
     update();
